;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Project 1: A Jaguar Shooter
;;
;; (c) reboot 2009
;; 
;; A Jagware Game
;;
;;


                .include    "jaguar.inc"
                .text
		
				include "_EQUATE.S"
				
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Code entrypoint
;;

top_of_the_code:
				lea		$802000,a0							; base of ROM address space
.find:			cmp.w	#'DI',(a0)+							; find the samples
				bne.s	.find
				cmp.w	#'GI',(a0)+
				bne.s	.find
				move.l	a0,$1008							; sample base

;				move.l	#140,$1100							; music vol
;				move.l	#176,$1104							; sample vol
;				move.l	#'0000',$1108						; hiscore
;				move.l	#'0000',$110c
;				move.l	#2,$1110							; difficulty (0-hard,1-normal,2-easy)		

				move.w	#%0000011011000111,VMODE			; 320x256!!
			
                jsr     InitVideo               			; Setup our video registers.
				jsr		Init_music

				bsr     ConvertBMPtoJAG						; Convert GFX assets to Jag format
				bsr		Create_TilePointerTable				; Create index of tile pointers
				bsr     MakeObjectList          			; Create bit magled list for OP
				bsr		install_GPU_code					; Copy the GPU code to GPU local memory
				move.l	$1110,difficulty_shifter			; Read the difficulty level
				bsr		setup_scoreboard					; Init score panel
	
				move.w	#map_entrypoint,map_y_cnt_save		; set up where on the map we start at
				move.w	#map_entrypoint,map_y_pos_save
				move.w	#-1,map_y_dir_save					; its scroll direction
				move.l	#startwave,wave_event_save		; the first wavetrigger definition
				move.l	#0,gun_powerlevel					; reset player weapon
				
				bsr		initialise_gameobjects				; setup all initial objects		

				lea		GPU_tilemap_update,a0				; update the map 
				bsr		call_GPU_code
				lea		GPU_save_and_plot,a0				; init the starfield
				bsr		call_GPU_code

				lea		$f00400,a1							; clear the pallette
				move.l	#255,d7
.pal:			move.w	#0,(a1)+
				dbra	d7,.pal
				
                move.l  #VB_INTERUPT,LEVEL0     			; Install 68K LEVEL0 handler
                move.w  a_vde,d0                			; Must be ODD
                sub.w   #16,d0
                ori.w   #1,d0
                move.w  d0,VI

                move.w  #1,INT1                 			; Enable video interrupts
                and.w   #$f8ff,sr

                bsr     wait_frame              			; Stop the OP from freaking
				
                move.l  #ob_list,d0
                swap    d0
                move.l  d0,OLP                  			; Object Processor is GO!

reset_game:		move.l	$1110,d0							; get difficulty level
				addq.l	#3,d0								; add 3
				move.l	d0,lives							; and thats our initial lives counter
				
deathloop:		move.w	#-1,shield_stayon					; not forced on
				move.w	#-1,shield_recharge					; can enable
				move.w	#-1,shield_active					; disabled
				clr.w	shield_wason						; wasn't on
				
				move.l	#0,__MIDBOSS__+boss_active
				move.l	#0,__ENDBOSS__+boss_active
				
				lea		bullets,a0
				move.l	#max_bullets-1,d7
.clr_bullets:	move.l	#-1,bullet_use(a0)
				lea		32(a0),a0
				dbra	d7,.clr_bullets

				move.l	#-1,s_buffer						; terminate score buffer
				sub.l	#1,lives							; subtract 1 from lives
				bpl		noquit								; if positive, not game over
				
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; game over
;;

game_over:		move.l	ship_obj,a0							; address of 1st
				move.l	a0,-(a7)
				move.l	#gameover_bmp,a1
				move.l  #bitmapobject,d0        			; type
				move.l  #41,d1                  			; height
                move.l  #320*2,d2              				; bytes to next line
                move.l  #4,d3                   			; 16-Bit
                move.l  #0,d4                   			; Non Transparent
                move.l  #320*2,d5              				; bytes/pixels*width / line;
				jsr     CreateObject	

				move.l	a0,-(a7)
				move.l	#gameover_txt_bmp,a1
				move.l  #bitmapobject,d0        			; type
				move.l  #8,d1                  				; height
                move.l  #40,d2              				; bytes to next line
                move.l  #0,d3                   			; 1-Bit
                move.l  #1,d4                   			; Transparent
                move.l  #40,d5              				; bytes/pixels*width / line;
				jsr     CreateObject

				lea		$0.w,a0
				lea		.gameover_txt,a1					; text output
				bsr		printer
				
				move.l	(a7)+,a0							; position textbox
				move.l	#60,d0
				move.l	#165,d1
				bsr		setup_object_xy
				
				move.l	(a7)+,a0							; position bitmap
				move.l	#16,d0
				move.l	#122,d1
				bsr		setup_object_xy
																						
.now_what:		jsr		wait_frame
				jsr		readpad								; read jagpad
				jsr		handle_input
				tst.w	dpad_a
				bne		.try_again
				tst.w	dpad_c
				beq		.now_what
				
				move.l	#1,kill_dsp							; shut down DSP semaphore
				jmp		$3000								; call the loader to re-load titles
			
.gameover_txt:	dc.l	gameover_txt_bmp
				dc.b	"Press A to retry, C for titles",-1
				even
			
.try_again:		
				move.l	a0,ship_obj							; restore the objects we trashed for Game Over
				move.l	#playership_bmp,a1
				move.l  #bitmapobject,d0        			; type
				move.l  #24,d1                  			; height (32)
                move.l  #(24*2)*3,d2             			; bytes to next line
                move.l  #4,d3                   			; 16-Bit
                move.l  #1,d4                   			; Transparent
                move.l  #24*2,d5              				; bytes/pixels*width / line;
                jsr     CreateObject
				
				move.l	a0,fujishld_obj						
				move.l	#fujishld_bmp,a1
				move.l  #bitmapobject,d0        			; type
				move.l  #48,d1                  			; height
                move.l  #48*2,d2   	         				; bytes to next line
                move.l  #4,d3                   			; 16-Bit
                move.l  #1,d4                   			; Transparent
                move.l  #48*2,d5              				; bytes/pixels*width / line;
                jsr     CreateObject
				
				move.l	#'0000',score_t						; BCD zero score
				move.l	#'0000',score_t+4
				move.w	#map_entrypoint,map_y_cnt_save		; reset scroll point
				move.w	#map_entrypoint,map_y_pos_save
				move.w	#-1,map_y_dir_save					; reset direction
				move.l	#startwave,wave_event_save		; reset checkpoints
				move.l	#0,gun_powerlevel					; reset gun power
				clr.w	have_shield							; reset pickups
				clr.w	have_sideshot
				clr.w	have_laser
				move.w	#1,have_sbomb
				
				bsr		initialise_gameobjects				; set up all objects again

				move.w	#160+64,ship_x						; player ship x
				move.w	#180+32,ship_y						; player ship y

				bra		reset_game					

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; You Win (Top of map reached)
;;
				
end_of_map:
				move.l	$1008,a0							; play "level complete" sample
				add.l	#complete_off,a0
				lea		$0,a1
				move.l	#complete_len,d0					; sample length
				moveq	#0,d1								; looplength (unlooped)
				moveq	#5,d2								; priority (high)
				move.l	$1104,d3							; volume
				moveq	#0,d4								; freq
				jsr		musicplayer+$c
				
			
				move.l	#5*50,d7
				move.l	#0,d6
.big_pause:		movem.l	d6/d7,-(a7)
				bsr		wait_frame

				lea		GPU_tilemap_update,a0
				bsr		call_GPU_code
				lea		GPU_restore_backdrop,a0
				bsr		call_GPU_code						; restore the backdrop (unplot starfield)
				lea		GPU_save_and_plot,a0
				bsr		call_GPU_code						; plot starfield (save the backdrop)
				bsr		thread_processing					; handle the enemy threads
				
				movem.l	(a7)+,d6/d7

				subq.w	#1,d6
				bpl.s	.noexpld
				moveq	#20,d6
				movem.l	d0-d7/a0-a6,-(a7)
				move.l	#_ENDBOSS_DEATH_,wave_event
				moveq	#0,d1
				move.w	map_y_pos,d1
				move.l	d1,_ENDBOSS_DEATH_
				bsr		trigger_processing
				movem.l	(a7)+,d0-d7/a0-a6
.noexpld:		dbra	d7,.big_pause

				bra		game_over							; then do normal game over
		
noquit:			tst.l	gun_powerlevel						; lower gun power level by 1
				beq.s	.at_base
				sub.l	#1,gun_powerlevel
.at_base:		move.l	gun_powerlevel,d0
				lea		new_bullet_power,a0
				add.l	d0,d0
				add.l	d0,d0
				move.l	(a0,d0.w),current_bullet_power
				
				bsr		initialise_gameobjects				; reset gameobjects
				lea		GPU_tilemap_update,a0
				bsr		call_GPU_code						; do the GPU stuffs (well, some of it!)
				lea		GPU_bullet_sort,a0
				bsr		call_GPU_code						; do the GPU bullet sorting
				bsr		draw_lives							; draw the lives on the panel
				
				move.l	$1008,a0							; "Get Ready!" sample
				add.l	#get_ready_off,a0
				lea		$0,a1
				move.l	#get_ready_len,d0					; sample length
				moveq	#0,d1								; looplength (unlooped)
				moveq	#10,d2								; priority (high)
				move.l	$1104,d3							; volume
				moveq	#0,d4								; freq
				jsr		musicplayer+$c
				
				lea		game_pal,a1							; fade in the game pallette
				bsr		fadepal_up

				move.w	#160+64,ship_x						; player ship x
				move.w	#180+32,ship_y						; player ship y

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Main game loop
;;				

gameloop:		bsr wait_frame								; vsync

				lea		GPU_tilemap_update,a0
				bsr		call_GPU_code						; update the map
															; process the bullets
															; move player ship
															; paralax object update
				lea		GPU_bullet_sort,a0
				bsr		call_GPU_code						; y-sort the bullets

				lea		GPU_restore_backdrop,a0
				bsr		call_GPU_code						; restore the backdrop (unplot starfield)
				lea		GPU_save_and_plot,a0
				bsr		call_GPU_code						; plot starfield (save the backdrop)
				
				bsr		trigger_processing					; check if on a trigger line
				bsr		thread_processing					; handle the enemy threads
				bsr		update_scoreboard					; do the score stuffs
								
				move.l	#-1,bspawn_table
				lea		GPU_Collision_Detection,a0
				bsr		call_GPU_code						; check if anything hit anything

				move.l	#playership_bmp,ship_anim			; base frame for player ship
				
				tst.l	explode_sfx_trigger					; did anything blow up?
				beq		.no_explosion

				tst.w	explode_sfx_timer
				bpl		.no_explosion
				move.w	#25,explode_sfx_timer				; (100)		
				
				move.l	$1008,a0							; "boom!"
				add.l	#explode_off,a0
				lea		$0,a1
				move.l	#17143,d0							; sample length
				moveq	#0,d1								; looplength (unlooped)
				moveq	#20,d2								; priority (high)
				move.l	$1104,d3							; volume
				moveq	#0,d4								; freq
				jsr		musicplayer+$c				
				clr.l	explode_sfx_trigger
	
.no_explosion:	tst.w	shield_active						; have we got a shield?
				beq.s	.vuln						
				
				move.l	#150,life_energy					; if so, top up energy
				clr.l	player_hit_flag						; and clear collision flag

.vuln:
				tst.l	player_hit_flag						; did we hit anything?
				beq.s	.not_hit
				clr.l	player_hit_flag
				add.l	#24*3*(48*3),ship_anim				; offset to hitframes in player bitmap [add.l #32*3*(64*3),ship_anim for 32x32]
				
.not_hit:		tst.l	life_energy							; still alive?
				bpl		.still_alive

				move.l	#-2,life_energy						; clear lifebar
				bsr		update_lifebar
				move.l	#150,life_energy					; reset energy levels

;; testing check
;;
;				bra		.still_alive
;;
;;				
			
				move.l	$1008,a0							; "ship dead sample"
				add.l	#shipdead_off,a0
				lea		$0,a1
				move.l	#12501,d0							; sample length
				moveq	#0,d1								; looplength (unlooped)
				moveq	#40,d2								; priority (high)
				move.l	$1104,d3							; volume
				moveq	#0,d4								; freq
				jsr		musicplayer+$c	
				
				moveq	#9,d7								; 10 frames in death "animation"
				lea		.death_anims,a6
.dloop:			movem.l	d7/a6,-(a7)	
				moveq	#1,d6
.frames:		bsr 	wait_frame							; vsync
				dbra	d6,.frames
				movem.l	(a7)+,d7/a6
				moveq	#0,d0
				moveq	#0,d1
				move.w	ship_x,d0
				sub.w	map_x_pos,d0						; get ship x pos
				move.w	ship_y,d1							; get ship y-pos
				move.l	(a6)+,d2							; get bitmap offset
				add.l	#playership_bmp,d2
				move.l	ship_obj,a0
				movem.l	d7/a6,-(a7)	
				bsr		setup_object_xyg					; set player ship object
				movem.l	(a7)+,d7/a6
				dbra	d7,.dloop
				
				move.l	ship_obj,a0
				move.l	#playership_bmp,a1
				move.l  #bitmapobject,d0        			; type
				move.l  #64,d1                  			; height (32)
                move.l  #(64*2),d2      	       			; bytes to next line
                move.l  #4,d3                   			; 16-Bit
                move.l  #1,d4                   			; Transparent
                move.l  #64*2,d5              				; bytes/pixels*width / line;
                jsr     CreateObject

				moveq	#7,d7								; 10 frames in death "animation"
				lea		.death_anims2,a6
.dloop2:		movem.l	d7/a6,-(a7)	
				moveq	#1,d6
.frames2:		bsr 	wait_frame							; vsync
				dbra	d6,.frames2
				movem.l	(a7)+,d7/a6
				moveq	#0,d0
				moveq	#0,d1
				move.w	ship_x,d0
				sub.w	#20,d0
				sub.w	map_x_pos,d0						; get ship x pos
				move.w	ship_y,d1							; get ship y-pos
				sub.w	#20,d1
				move.l	(a6)+,d2							; get bitmap offset
				move.l	ship_obj,a0
				movem.l	d7/a6,-(a7)	
				bsr		setup_object_xyg					; set player ship object
				movem.l	(a7)+,d7/a6
				dbra	d7,.dloop2
				
				move.l	ship_obj,a0
				move.l	#playership_bmp,a1
				move.l  #bitmapobject,d0        			; type
				move.l  #24,d1                  			; height (32)
                move.l  #(24*2)*3,d2             				; bytes to next line
                move.l  #4,d3                   			; 16-Bit
                move.l  #1,d4                   			; Transparent
                move.l  #24*2,d5              				; bytes/pixels*width / line;
                jsr     CreateObject
				
				move.l	#9000,d0							; set ship offscreen
				move.w	d0,ship_x
				
				move.l	e_thread4+enemy_obj,a0
				move.l	d0,d1
				bsr		setup_object_xy						; set any bonus pickups offscreen (they are 16bit and wont fade)
				move.l	#-1,thread4+thread_active			; kill thread 4 (bonus thread)
				
				tst.l	lives								; last life?
				bne.s	.no_over
				
				move.l	$1008,a0							; "Game Over!"
				add.l	#game_over_off,a0
				lea		$0,a1
				move.l	#game_over_len,d0					; sample length
				moveq	#0,d1								; looplength (unlooped)
				moveq	#10,d2								; priority (high)
				move.l	#250,d3								; ignore volume level (always say GAME OVER)
				moveq	#0,d4								; freq
				jsr		musicplayer+$c
				bra		.is_over
		
.no_over:		bsr     fadepal_down						; fade down the playscreen
				
.is_over:		lea		GPU_restore_backdrop,a0				; unplot the starfield
				bsr		call_GPU_code
				
				bra		deathloop							; reset game to last checkpoint
				
;;.death_anims:	dc.l	192*(64*3),(192*(64*3))+64,(192*(64*3))+128		;(32x32) offsets for Fuji Fighter bitmaps
;;				dc.l	224*(64*3),(224*(64*3))+64,(224*(64*3))+128
;;				dc.l	256*(64*3),(256*(64*3))+64,(256*(64*3))+128
;;				dc.l	288*(64*3)

.death_anims:	dc.l	144*(48*3),(144*(48*3))+48,(144*(48*3))+96		;(24x24) offsets for Fuji Fighter bitmaps
				dc.l	168*(48*3),(168*(48*3))+48,(168*(48*3))+96
				dc.l	192*(48*3),(192*(48*3))+48,(192*(48*3))+96
				dc.l	216*(48*3)
				
.death_anims2:	dc.l	shipdead_bmp
				dc.l	shipdead_bmp+(8192*1)
				dc.l	shipdead_bmp+(8192*2)
				dc.l	shipdead_bmp+(8192*3)
				dc.l	shipdead_bmp+(8192*4)
				dc.l	shipdead_bmp+(8192*5)
				dc.l	shipdead_bmp+(8192*6)
				dc.l	shipdead_bmp+(8192*7)

.still_alive:	clr.l	player_hit_flag						; clear collision flag

				tst.l	bonus_hit_flag						; hit a bonus item?
				beq		.no_bonus_pickup
				
				bsr 	bonus_processing					; process it!
				
.no_bonus_pickup:
				jsr		readpad								; read jagpad
				jsr		handle_input						; process jagpad
				
.pause_hold:	tst.w	PAUSE_ON							; pause pressed?
				beq.s	.check_reset
				
				bsr		pause_mode							; BAAAAAAAAAAAAAAAAAAAAAD MOOOOOOOOOOOOOOODIES!!!!

.check_reset:	tst.w	exit_flag							; *# ?
				beq.s	.check_up
				
.die_gorf_die:	jmp		$3000								; reboot the titles
				
.check_up:		tst.w	dpad_up								; DPAD up?
				beq		.check_down
				add.l	#24*(48*3),ship_anim				; offset to UP row of animations [32x32:add.l #32*(64*3),ship_anim]
				cmp.w	#72,ship_y							; top of screen?
				blt		.check_left
				sub.w	#2,ship_y							; move ship up
				bra		.check_left
				
.check_down:	tst.w	dpad_down							; DPAD down?
				beq		.check_left
				add.l	#48*(48*3),ship_anim				; offset to DOWN row of animations [32x32:add.l	#64*(64*3),ship_anim]
				cmp.w	#220,ship_y							; bottom of screen?
				bgt		.check_left
				add.w	#2,ship_y							; move ship down
				
.check_left:	tst.w	dpad_left							; DPAD left?
				beq		.check_right
				add.l	#48,ship_anim						; offset to bank left [32x32:add.l #64,ship_anim]

				move.w	#20,d0								; left edge of screen?
				cmp.w	ship_x,d0
				bgt		.joystick_done
				sub.w	#4,ship_x							; move left
				
				cmp.w	#2,map_x_pos						; max far left?
				blt		.joystick_done
				
				cmp.w	#288-16,ship_x						; ship in scroll zone?
				bgt		.joystick_done
				
				sub.w	#2,ship_x							; move ship
				sub.w	#2,map_x_pos						; adjust map
				subq.l	#1,px_now							; adjust parallax layer
				bne		.joystick_done
				move.l	#2,px_now							; parallax layer far left
				add.l	#2,px_start
				bra		.joystick_done
				
.check_right:	tst.w	dpad_right							; as above, but for right
				beq		.joystick_done
				

				move.w	#(mapwidth*32)-56,d0
.palskipr:
				add.l	#24*4,ship_anim						; [32x32: add.l	#128,ship_anim]
				cmp.w	ship_x,d0
				blt		.joystick_done
				add.w	#4,ship_x
				
				cmp.w	#86,map_x_pos						; edge of map?
				bgt.s	.joystick_done
							
				cmp.w	#128,ship_x							; in scroll zone
				blt.s	.joystick_done
				
				add.w	#2,ship_x
							
				add.w	#2,map_x_pos
				subq.l	#1,px_now
				bne.s	.joystick_done
				move.l	#2,px_now				
				sub.l	#2,px_start

				
.joystick_done:
				move.w	#0,shield_active					; turn shield flag off
				move.l	#700,d0								; x-shield is offscreen
				move.l	d0,d1								; y-shield is offscreen
				
				tst.w	have_shield							; got shield?
				beq		.set_shield_xy						; and set off if you dont have it
				
				tst.w	shield_stayon						; must shield stay on?
				bpl		.force_shield_on					; if so, force it on
			
				tst.w	shield_recharge						; shield recharging?
				bpl		.check_not_pressed
			
				tst.w	dpad_three							; check button 3
				beq 	.check_not_pressed					; if not pressed turn off
.check_pressed:
				tst.w	shield_active						; was shield already on?
				bne		.no_on_sampl
				
				movem.l	d0-d1,-(a7)							; "Shield Activated"
				move.l	$1008,a0
				add.l	#shield_activated_off,a0
				lea		$0,a1
				move.l	#shield_activated_len,d0			; sample length
				moveq	#0,d1								; looplength (unlooped)
				moveq	#10,d2								; priority (high)
				move.l	$1104,d3							; volume
				moveq	#0,d4								; freq
				jsr		musicplayer+$c
				movem.l	(a7)+,d0-d1
				
.no_on_sampl:
				move.w	#15*50,shield_recharge				; 15 seconds to recharge after drop
				move.w	#5*50,shield_stayon					; ^ 5 active + 10 downtime
				move.w	#1,shield_wason						; flag shield as "was on"
				bra		.force_shield_on
.check_not_pressed:
				tst.w	shield_stayon						; must shield stay on?
				bpl		.force_shield_on
				
				tst.w	shield_wason						; and was it on before?
				beq		.set_shield_xy						; skip sample
				clr.w	shield_wason
				
				movem.l	d0-d1,-(a7)							; "Shield Deactivated"
				move.l	$1008,a0
				add.l	#shield_deactivated_off,a0
				lea		$0,a1
				move.l	#shield_deactivated_len,d0			; sample length
				moveq	#0,d1								; looplength (unlooped)
				moveq	#10,d2								; priority (high)
				move.l	$1104,d3							; volume
				moveq	#0,d4								; freq
				jsr		musicplayer+$c
				movem.l	(a7)+,d0-d1

				bra		.set_shield_xy
				
.force_shield_on:
				move.w	#1,shield_active					; shield is on!
				move.w	ship_x,d0							; get ship co-ords
				sub.w	map_x_pos,d0
				move.w	ship_y,d1							; and adjust for the shield position
				sub.l	#12,d0
				subq	#8,d1
				
				sub.w	#1,shield_timer						; how long left till deactivate?
				bpl		.set_shield_xy
				
				clr.w	shield_active						; shield all gone
				clr.w	have_shield
				move.l	#700,d0								; remove icon from top panel
				move.l	d0,d1
				move.l	shldobj,a0
				bsr		setup_object_xy
				
.set_shield_xy:	move.l	fujishld_obj,a0						; shield object address

				move.l	.next_shield,a2						; get shield animation frame address
.get:			move.l	(a2)+,d2							; animation wrapped?
				bpl.s	.s_ok
				lea		.shield_table,a2					; reset animation
				bra.s	.get
.s_ok:			move.l	a2,.next_shield
				
				bsr		setup_object_xyg					; draw shield around ship

				tst.l	resume_scrolling					; normal or boss mode?
				bmi		.boss_core_logic

				sub.w	#1,map_y_cnt						; scroll map one scanline
				bmi		end_of_map							; check if at end?
			
				move.w	map_y_dir,d0
				add.w	d0,map_y_pos
				bsr 	paralax								; adjust the parallax layer
				bra		.fire_1								

.next_shield:	dc.l	.shield_table
.shield_table:	dc.l	fujishld_bmp+0
				dc.l	fujishld_bmp+0
				dc.l	fujishld_bmp+4608
				dc.l	fujishld_bmp+4608
				dc.l	fujishld_bmp+(4608*2)
				dc.l	fujishld_bmp+(4608*2)
				dc.l	fujishld_bmp+(4608*3)
				dc.l	fujishld_bmp+(4608*3)
				dc.l	fujishld_bmp+(4608*4)
				dc.l	fujishld_bmp+(4608*4)
				dc.l	fujishld_bmp+(4608*5)
				dc.l	fujishld_bmp+(4608*5)
				dc.l	fujishld_bmp+(4608*6)
				dc.l	fujishld_bmp+(4608*6)
				dc.l	fujishld_bmp+(4608*7)
				dc.l	fujishld_bmp+(4608*7)
				dc.l	-1
				
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Boss Encounter code
;;
;;

.boss_flash_active:		dc.w	0							; is boss flashing? (hit)					
.boss_flash_pointer:	dc.l	.boss_flash_colours
.boss_flash_colours:	incbin  "ENEMIES/MIDBOSS/BOSSPAL.PAL"	
						dc.w	"RB"
.boss_core_logic:
				cmp.l	#7,e_thread1+enemy_action2			; was boss hit?
				bne.s	.no_boss_fresh_hit

				tst.w	.boss_flash_active					; boss already flashing?
				bmi.s	.no_boss_fresh_hit
				move.w	#-1,.boss_flash_active				; activate flash
				
.no_boss_fresh_hit:
				tst.w	.boss_flash_active					; is boss flashing?
				beq		.no_boss_flash_active				; if not, continue as normal
				
				move.l	.boss_flash_pointer,a0				; get next colour in flash
				move.w	(a0)+,d0
				cmp.w	#'RB',d0
				bne.s	.set_boss_pal						; end of flash colours?
				clr.w	.boss_flash_active					; disable flash
				move.l	#.boss_flash_colours,.boss_flash_pointer	; reset pointer
				bra		.no_boss_flash_active
.set_boss_pal:	move.w	d0,$f00400+(32*15)+30				; flash next colour to register
				move.l	a0,.boss_flash_pointer				; update pointer
				
.no_boss_flash_active:
				moveq	#0,d7								; d7 =-1 at end, then no resume

.test_t0:		tst.l	kill_wavet0							; thread0 must be stopped?
				beq		.test_t1
				tst.l	thread1+thread_active
				bmi		.test_t1							; if it is, then skip to next
				moveq	#-1,d7								; restart FAILED
				bra		.core_test_done
.test_t1:		tst.l	kill_wavet1							; thread1 must be stopped?
				beq		.test_t2
				tst.l	thread2+thread_active
				bmi		.test_t2							; if it is, then skip to next
				moveq	#-1,d7								; restart FAILED
				bra		.core_test_done
.test_t2:		tst.l	kill_wavet2							; thread2 must be stopped?
				beq		.test_t4
				tst.l	thread3+thread_active
				bmi		.test_t4							; if it is, then skip to next
				moveq	#-1,d7								; restart FAILED
				bra		.core_test_done
.test_t4:		tst.l	kill_wavet3							; thread4 must be stopped?
				beq		.test_t5
				tst.l	thread5+thread_active
				bmi		.test_t5
				moveq	#-1,d7
				bra		.core_test_done
.test_t5:		tst.l	kill_wavet4							; thread5 must be stopped?
				beq		.core_test_done
				tst.l	thread6+thread_active
				bmi		.core_test_done
				moveq	#-1,d7
.core_test_done:
				tst.l	d7									; should we resume scrolling?
				bne.s	.fire_1
				move.l	#0,resume_scrolling					; yes!
				
.fire_1:
				bsr		update_lifebar						; update the energy bar
				tst.w	dpad_b
				bne		.fire_gun							; button b
				tst.w	dpad_a
				bne		.fire_laser							; button a
				tst.w	dpad_c
				bne		.fire_smartbomb						; button c
				bra		gameloop
				
.fire_smartbomb:
				tst.w	have_sbomb							; smartbomb collected?
				beq		gameloop
				
				clr.w	have_sbomb							; remove it from the panel
				move.l	bombobj,a0
				move.l	#700,d0
				move.l	#700,d1
				bsr		setup_object_xy

				movem.l	d0-d7/a0-a6,-(a7)
				
				move.l	$1008,a0							; smartbomb explode sample
				add.l	#smartbomb_fire_off,a0
				lea		$0,a1
				move.l	#smartbomb_fire_len,d0				; sample length
				moveq	#0,d1								; looplength (unlooped)
				moveq	#5,d2								; priority (high)
				move.l	$1104,d3							; volume
				moveq	#0,d4								; freq
				jsr		musicplayer+$c
				movem.l	(a7)+,d0-d7/a0-a6
				
				move.l	#0,sbomb_type						; sbomb_type:0 [normal smartbomb]
				lea		GPU_smartbomb,a0
				bsr		call_GPU_code						; execute smartbomb GPU module
				
				bra		gameloop
				
.fire_laser:	tst.w	have_laser							; laser collected?
				beq		gameloop
				
				tst.w	shoot_delay							; can we fire again yet?
				bpl		gameloop
				
				sub.w	#1,laser_counter					; subtract one round
				bpl.s	.still_got_it
				
				clr.w	have_laser							; remove the icon from the panel
				move.l	lasrobj,a0
				move.l	#700,d0
				move.l	#700,d1
				bsr		setup_object_xy
				
				movem.l	d0-d7/a0-a6,-(a7)					; "Laser Depleated!"
				move.l	$1008,a0
				add.l	#laser_deactivated_off,a0
				lea		$0,a1
				move.l	#laser_deactivated_len,d0			; sample length
				moveq	#0,d1								; looplength (unlooped)
				moveq	#10,d2								; priority (high)
				move.l	$1104,d3							; volume
				moveq	#0,d4								; freq
				jsr		musicplayer+$c
				movem.l	(a7)+,d0-d7/a0-a6
				
.still_got_it:
				move.w	#bullet_reload,shoot_delay			; reset reload delay

				move.w	ship_x,d0							; get x/y of ship
				move.w	ship_y,d1							; and adjust for laser position
				sub.w	#2,d0				
				add.w	#4,d1
				
				moveq	#7,d7								; 8 bullets in laser stream
._las:			move.l	#1,new_bullet_owner
				move.l	#13,new_bullet_type
				move.w	d0,new_bullet_x+2
				move.w	d1,new_bullet_y+2
				lea		add_bullet_GPU,a0					; GPU update bullet table!
				movem.l	d0/d1/d7,-(a7)
				bsr		call_GPU_code	
				movem.l	(a7)+,d0/d1/d7
				subq	#4,d1								; decrement Y position
				dbra	d7,._las

				move.l	$1008,a0							; "Jack The Dog - Sneezing!"
				add.l	#jack_off,a0
				lea		$0,a1
				move.l	#5677,d0							; sample length
				moveq	#0,d1								; looplength (unlooped)
				moveq	#20,d2								; priority (high)
				move.l	$1104,d3							; volume
				moveq	#0,d4								; freq
				jsr		musicplayer+$c
				
				bra		gameloop
				
.fire_gun:
				tst.w	shoot_delay							; gun reloaded?
				bpl		gameloop
				move.w	#bullet_reload,shoot_delay			; reset gun delay

				move.l	P_00,d0								; rotate the next bullet bitmap
				add.l	#32,d0								; into the bullet type table
				cmp.l	#bullet00_bmp+(32*7),d0				; so the bullet colour
				bne.s	.no_wrap							; cycles as you shoot
				move.l	#bullet00_bmp,d0
.no_wrap:		move.l	d0,P_00
				move.l	d0,P_01
				move.l	d0,P_02
				move.l	d0,P_03
				move.l	d0,P_04
				
				cmp.w	#1,have_sideshot					; do we have a sideshot?
				bne.s	.no_sideshot
			
				move.l	#1,new_bullet_owner					; bullet is "ours"
				move.l	#14,new_bullet_type					; sideshot type
				move.w	ship_x,new_bullet_x+2
				move.w	ship_y,new_bullet_y+2
				sub.w	#2,new_bullet_x+2					; offset from ship for sideshot
				add.w	#16,new_bullet_y+2
				lea		add_bullet_GPU,a0					; GPU update bullet table!
				bsr		call_GPU_code				
			
.no_sideshot:
				cmp.l	#'DUAL',current_bullet_power		; dual shot mode? (powerup #2)
				beq		.dual_shot
				
				move.l	#1,new_bullet_owner					; bullet is "ours"
				move.l	current_bullet_power,new_bullet_type	; get powerup level
				move.w	ship_x,new_bullet_x+2
				move.w	ship_y,new_bullet_y+2
				sub.w	#2,new_bullet_x+2					; offset to shot from
				add.w	#4,new_bullet_y+2
				lea		add_bullet_GPU,a0					; GPU update bullet table!
				bsr		call_GPU_code
				bra		.play_gunshot
	
.dual_shot:		move.l	#1,new_bullet_owner					; bullet is ours
				move.l	#0,new_bullet_type
				move.w	ship_x,new_bullet_x+2				; offset from ship to shoot from
				move.w	ship_y,new_bullet_y+2
				add.w	#4,new_bullet_y+2
				lea		add_bullet_GPU,a0					; GPU update bullet table!
				bsr		call_GPU_code
				sub.w	#5,new_bullet_x+2					; adjust offset
				lea		add_bullet_GPU,a0					; GPU update bullet table!
				bsr		call_GPU_code
		
.play_gunshot:	move.l	$1008,a0							; "zzap!"
				add.l	#fastgun_off,a0
				lea		$0,a1
				move.l	#3578,d0							; sample length
				moveq	#0,d1								; looplength (unlooped)
				moveq	#20,d2								; priority (high)
				move.l	$1104,d3							; volume
				moveq	#0,d4								; freq
				jsr		musicplayer+$c
				
                bra     gameloop							; and repeat, to infinity
				
paralax:		subq.l	#1,py_now							; y-wrap on bitmap?
				bne.s	.set_x
				move.l	#2,py_now							; scroll down!
				sub.l	#240,py_start					
				sub.l	#1,py_cnt							; time to wrap?
				bne.s	.set_x
				move.l	#underlay_bmp,py_start				; reset parallax bitmap
				add.l	#240*256,py_start
				move.l	#256,py_cnt							; [GPU LOOP WILL DO THE ACTUAL OBJECT UPDATE]
.set_x:			rts

update_lifebar:
				move.l	lifebar_obj,a0						; address of object
				move.l	#150,d2								; maximum health
				sub.l	life_energy,d2						; subtract how much we've been hit
				add.w	d2,d2								; *2 (bitmap is 300 long)
				add.l	#lifebar_bmp,d2						; add that on
				move.w	ebar_x,d0
				move.w	ebar_y,d1
				bsr		setup_object_xyg					; and draw the lifebar

				tst.w	ebar_ticks							; time to regen?
				bpl.s	.no_add
				cmp.l	#150,life_energy					; at max energy?
				beq.s	.no_add
				add.l	#1,life_energy						; increment energybar
				move.w	#25,ebar_ticks
.no_add:		rts

				.phrase
player_bullet:	dc.l	0

draw_lives:		move.l	lives,d0							; get lives remaining
				asl.w	#2,d0								; *4
				move.l	.lives_table(pc,d0.w),a6			; offset into table
				lea		lives_obj,a5
				moveq	#3,d7								; we have 4 to plot
.update_lives:	moveq	#0,d0
				moveq	#0,d1
				move.w	(a6)+,d0							; get x
				move.w	(a6)+,d1							; get y
				move.l	(a5)+,a0							; get object address
				movem.l	d7/a5-a6,-(a7)
				bsr		setup_object_xy						; set up object (either in panel or offscreen)
				movem.l	(a7)+,d7/a5-a6
				dbra	d7,.update_lives
				rts

.lives_table:	dc.l	.l_00,.l_01,.l_02,.l_03,.l_04

.l_00:			dc.w	700,700,700,700,700,700,700,700		; all offscreen
.l_01:			dc.w	700,700,700,700,700,700,310,036
.l_02:			dc.w	700,700,700,700,278,036,310,036
.l_03:			dc.w	700,700,246,036,278,036,310,036
.l_04:			dc.w	214,036,246,036,278,036,310,036

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Trigger Processing
;;

					.phrase
killwave_flags:
kill_wavet0:		dc.l 0									
kill_wavet1:		dc.l 0
kill_wavet2:		dc.l 0
kill_wavet3:		dc.l 0
kill_wavet4:		dc.l 0
					.phrase
resume_scrolling:	dc.l 0
boss_death:			dc.l 0

wave_event:		dc.l	wavetriggers

flasherror:		add.l	#$00010001,$f00400
				bra.s	flasherror

trigger_processing:
				move.l	wave_event,a0
				move.l	(a0),d0
				moveq	#0,d1
				move.w	map_y_pos,d1
				cmp.w	d0,d1
				beq.s	.hit
				rts
.hit:			cmp.l	#'WAVE',84(a0)
				bne		flasherror
				add.l	#88,wave_event
	
.wavematch:
				cmp.l	#'SAVE',boss_flag(a0)	; check if its a save triggerpoint
				beq		.do_savepoint
				cmp.l	#'SAMP',boss_flag(a0)	; check if its a sample triggerpoint
				beq		.do_samplepoint
				cmp.l	#'KILL',boss_flag(a0)	; check if its a killall triggerpoint
				beq		.do_killall
				cmp.l	#'BOSS',boss_flag(a0)	; check if a bosswave triggerpoint
				bne		.normal_wavetype
				tst.l	boss_active(a0)
				beq		.do_boss
				rts
				
.do_killall:	
				move.l	#1,sbomb_type
				lea		GPU_smartbomb,a0
				bsr		call_GPU_code
				rts

.do_samplepoint:
				move.l	a0,a6
				move.l	$1008,a0
				add.l	boss_pointer(a6),a0		; get samplebase
				lea		$0,a1
				move.l	boss_killt0(a6),d0		; sample length
				moveq	#0,d1					; looplength (unlooped)
				moveq	#10,d2					; priority (high)
				move.l	$1104,d3				; volume
				moveq	#0,d4					; freq
				jsr		musicplayer+$c
				rts
				
.do_savepoint:
				move.w	map_y_cnt,map_y_cnt_save
				move.w	map_y_pos,map_y_pos_save
				move.w	map_y_dir,map_y_dir_save
				move.l	wave_event,wave_event_save
				
				move.l	$1008,a0
				add.l	#checkpoint_off,a0
				lea		$0,a1
				move.l	#checkpoint_len,d0	; sample length
				moveq	#0,d1		; looplength (unlooped)
				moveq	#10,d2		; priority (high)
				move.l	$1104,d3	; volume
				moveq	#0,d4		; freq
				jsr		musicplayer+$c
				
				rts
				
.do_boss:
				move.l	#1,boss_active(a0)		; boss_active!
				
				move.l	#-1,resume_scrolling	; All STOP!

				move.l	boss_patterns(a0),d0	; get number of waves in boss waveset
				subq.l	#1,d0
				
				movem.l	boss_killt0(a0),d1-d5	; get all killwave flags
				movem.l d1-d5,killwave_flags
				
				move.l	boss_deathproc(a0),boss_death	; and what to do when it's dead
				
				move.l	boss_pointer(a0),a0		; reset A0 to point to bosswave patterns
.setuploop:		move.l	a0,-(a7)
				move.l	d0,-(a7)
				bsr		.normal_wavetype
				move.l	(a7)+,d0
				move.l	(a7)+,a0
				add.l	#88,a0
				dbra	d0,.setuploop
				
				rts
								
; this routine only really wants a0=pointer to wavedata to setup, so we call call it
; from the boss trigger routine easily :)
;
				
.normal_wavetype:
				move.l	wave_enemytype(a0),d0	; get enemy type
				move.l	d0,d1
				lea		enemy_lookup,a6
				asl.w	#2,d0	
				move.l	(a6,d0.w),a1			; a1=Enemy Type lookup
				
				move.l	etype_thread(a1),d0		; get thread number
				move.w	d0,-(a7)				; save thread number
				lea		thread_lookup,a2
				asl.w	#2,d0
				move.l	(a2,d0.w),a2			; a2=pointer to thread defintiion table
				move.l	#1,thread_active(a2)	; set thread = active!
				move.l	d1,thread_etype(a2)
						
				move.l	(a2),a3					; a3=Pointer to thread per enemy (Finally)

				move.l	wave_enemycount(a0),d7	; number of enemies in wave
				move.l	d7,thread_ecounter(a2)	; store in thread data
				
				subq	#1,d7					; dec for dbra

				move.l	wave_trigline(a0),thread_reset(a2)
				
				move.l	wave_initdelay(a0),d5
						
				lea		wave_trackdata(a0),a5	; a5=pointer to first wavedata pointer
				moveq	#0,d6					; incrementer to next pointer
				moveq	#0,d1					; init delay for current enemy
				tst.l	wave_unique(a0)			; all on same wave?
				beq.s	.setup_loop
				moveq	#4,d6
.setup_loop:	move.l	(a5),enemy_waveptr(a3)	; store pointer to wave for this enemy
				move.l	d1,enemy_initdelay(a3)	; init delay
;				move.l	#0,enemy_curframe(a3)
				move.l	#1,enemy_use(a3)
				move.l 	etype_tilldead(a1),enemy_tildead(a3)
				move.l	#0,enemy_action(a3)
				move.l	etype_1stframe(a1),enemy_curframe(a3)
;				move.l	etype_animspd(a1),enemy_animspd(a3)
				move.l	32(a5),enemy_animspd(a3)
								
.init_next:		lea		64(a3),a3				; next enemy
				add.l	d6,a5					; offset to next wave
				add.l	d5,d1					; inc init delay
				dbra	d7,.setup_loop
				
				cmp.w	#3,(a7)+
				beq.s	.rts					; skip pallette set on BONUS WAVE

				move.l	etype_pallette(a1),a0
				move.l	etype_pallbank(a1),a1
				movem.l	(a0),d0-d7
				movem.l	d0-d7,(a1)
				
.rts:			rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Thread Processing
;;

thread_processing:
				tst.l	thread1+thread_active
				bpl.s	.setupGPUt1
.test_thd2:		tst.l	thread2+thread_active
				bpl.s	.setupGPUt2
.test_thd3:		tst.l	thread3+thread_active
				bpl.s	.setupGPUt3
.test_thd4:		tst.l	thread4+thread_active
				bpl.s	.setupGPUt4
.test_thd5:		tst.l	thread5+thread_active
				bpl.s	.setupGPUt5
.test_thd6:		tst.l	thread6+thread_active
				bpl.s	.setupGPUt6
				rts
				
.setupGPUt1:	lea		thread1_GPU,a0
				bsr 	call_GPU_code
				bsr		.bulletspawns
				bra.s	.test_thd2

.setupGPUt2:	lea		thread2_GPU,a0
				bsr		call_GPU_code
				bsr		.bulletspawns
				bra.s	.test_thd3
				
.setupGPUt3:	lea		thread3_GPU,a0
				bsr		call_GPU_code
				bsr		.bulletspawns
				bra		.test_thd4
				
.setupGPUt4:	lea		thread4_GPU,a0
				bsr		call_GPU_code
				bsr		.bulletspawns
				bra		.test_thd5
				
.setupGPUt5:	lea		thread5_GPU,a0
				bsr		call_GPU_code
				bsr		.bulletspawns
				bra		.test_thd6

.setupGPUt6:	lea		thread6_GPU,a0
				bsr		call_GPU_code
				bsr		.bulletspawns
				rts

				
.bulletspawns:	lea		bspawn_table,a6
				lea		add_bullet_GPU,a0					; GPU update bullet table!
.loop:			move.l	(a6)+,d0
				bmi.s	.exit

				move.l	d0,new_bullet_x
				move.l	(a6)+,new_bullet_y
				move.l	(a6)+,new_bullet_type
				move.l	#2,new_bullet_owner
				bsr		call_GPU_code
				bra.s	.loop
.exit:			rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; bonus pickup processing
;;

						.phrase
current_bullet_power:	dc.l	0		
gun_powerlevel:			dc.l	0
have_laser:				dc.w	0
have_sbomb:				dc.w	1
have_shield:			dc.w	0
have_sideshot:			dc.w	0
shield_timer:			dc.w	50*20		; 20 seconds!
laser_counter:			dc.w	24			; 25 shots
new_bullet_power:		dc.l	0
						dc.b    "DUAL"
						dc.l	7,8,8
shield_active:			dc.w	0
shield_wason:			dc.w	0
shield_stayon:			dc.w	0
shield_recharge:		dc.w	0
			
bonus_processing:
				clr.l	bonus_hit_flag			; flag as done
				
				move.l	bonus_hit_flag+4,d0		; get powerup frame number
				add.l	d0,d0
				add.l	d0,d0
				lea		.bonus_table(pc),a0
				move.l	(a0,d0.w),a0
				jmp		(a0)
				
;;											; Frame number
.bonus_table:	dc.l	.life_pickup		; 0
				dc.l	.life_pickup		; 1
				dc.l	.laser_pickup		; 2 
				dc.l	.laser_pickup		; 3
				dc.l	.laser_pickup		; 4
				dc.l	.laser_pickup		; 5
				dc.l	.shield_pickup		; 6
				dc.l	.shield_pickup		; 7
				dc.l	.shield_pickup		; 8
				dc.l	.shield_pickup		; 9
				dc.l	.shield_pickup		; 10
				dc.l	.shield_pickup		; 11
				dc.l	.shield_pickup		; 12
				dc.l	.shield_pickup		; 13
				dc.l	.smartbomb_pickup	; 14
				dc.l	.smartbomb_pickup	; 15
				dc.l	.smartbomb_pickup	; 16
				dc.l	.smartbomb_pickup	; 17
				dc.l	.side_pickup		; 18
				dc.l	.side_pickup		; 19
				dc.l	.gun_pickup			; 20
				dc.l	.gun_pickup			; 21
				dc.l	.gun_pickup			; 22
				dc.l	.gun_pickup			; 23
				dc.l	.boss_weapon		; 24
				
.boss_weapon:
				move.l	#1,player_hit_flag
				sub.l	#25,life_energy
				rts

.life_pickup:
				cmp.l	#4,lives
				beq.s	.samp_only
				add.l	#1,lives
				
				bsr		draw_lives

.samp_only:		move.l	$1008,a0
				add.l	#life_off,a0
				lea		$0,a1
				move.l	#life_len,d0	; sample length
				moveq	#0,d1		; looplength (unlooped)
				moveq	#10,d2		; priority (high)
				move.l	$1104,d3	; volume
				moveq	#0,d4		; freq
				jsr		musicplayer+$c

				rts
				
.side_pickup:
				move.w	#1,have_sideshot
				move.l	$1008,a0
				add.l	#sideshot_off,a0
				lea		$0,a1
				move.l	#sideshot_len,d0	; sample length
				moveq	#0,d1		; looplength (unlooped)
				moveq	#10,d2		; priority (high)
				move.l	$1104,d3	; volume
				moveq	#0,d4		; freq
				jsr		musicplayer+$c

				move.l	sideobj,a0
				move.l	#22,d0
				move.l	#32,d1
				bsr		setup_object_xy				
				rts
				
.gun_pickup:
				move.l	$1008,a0
				add.l	#blaster_upgrade_off,a0
;				lea		gun_up,a0	; sample
				lea		$0,a1
				move.l	#blaster_upgrade_len,d0	; sample length
				moveq	#0,d1		; looplength (unlooped)
				moveq	#10,d2		; priority (high)
				move.l	$1104,d3	; volume
				moveq	#0,d4		; freq
				jsr		musicplayer+$c	

				move.l	gun_powerlevel,d0
				cmp.l	#4,d0
				beq		.exit_gun
				add.l	#1,d0
				move.l	d0,gun_powerlevel
				lea		new_bullet_power,a0
				add.l	d0,d0
				add.l	d0,d0
				move.l	(a0,d0.w),current_bullet_power
				
.exit_gun:		rts
			
.laser_pickup:	move.w	#1,have_laser
				move.w	#24,laser_counter

				move.l	lasrobj,a0
				move.l	#22+16,d0
				move.l	#32,d1
				bsr		setup_object_xy

				move.l	$1008,a0
				add.l	#laser_upgrade_off,a0
;				lea		laser_up,a0	; sample
				lea		$0,a1
				move.l	#laser_upgrade_len,d0	; sample length
				moveq	#0,d1		; looplength (unlooped)
				moveq	#10,d2		; priority (high)
				move.l	$1104,d3		; volume
				moveq	#0,d4		; freq
				jsr		musicplayer+$c	
				
				rts
				
.smartbomb_pickup:
				move.w	#1,have_sbomb
				move.l	bombobj,a0
				move.l	#22,d0
				move.l	#48,d1
				bsr		setup_object_xy

				move.l	$1008,a0
				add.l	#smartbomb_upgrade_off,a0
;				lea		smart_up,a0	; sample
				lea		$0,a1
				move.l	#smartbomb_upgrade_len,d0	; sample length
				moveq	#0,d1		; looplength (unlooped)
				moveq	#10,d2		; priority (high)
				move.l	$1104,d3	; volume
				moveq	#0,d4		; freq
				jsr		musicplayer+$c	
				
				rts
				
.shield_pickup:
				move.w	#1,have_shield
				move.w	#50*20,shield_timer
				move.l	shldobj,a0
				move.l	#22+16,d0
				move.l	#48,d1
				bsr		setup_object_xy

				move.l	$1008,a0
				add.l	#shield_off,a0
				lea		$0,a1
				move.l	#shield_len,d0	; sample length
				moveq	#0,d1		; looplength (unlooped)
				moveq	#10,d2		; priority (high)
				move.l	$1104,d3	; volume
				moveq	#0,d4		; freq
				jsr		musicplayer+$c	
				
				rts
				
				
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; set object pointed to by (a0) to new x/y position and bitmap in d2
;;
;;
;; a0=object list data
;; d0=x
;; d1=y
;; d2=bitmap address
;;
				
setup_object_xyg: 
				add.w	left_marg,d0
			
.offsdone:		and.w	#%111111111111,d0					; mask x value only (for negatives)
				and.w	#%1111111111,d1						; and again (y)
				move.w  6(a0),d4                			; get y-pos
               and.w   #$c007,d4
               move.w  14(a0),d6               			; get x-pos
               and.w   #$f000,d6
                add.w   d1,d1
                lsl.w   #3,d1
                or.w    d1,d4
                or.w    d0,d6
                move.w  d4,6(a0)							; update in object data
                move.w  d6,14(a0)

;						  01234567890123456789012345678912
				lsl.l	#8,d2
				move.l	(a0),d0
				and.l	#%00000000000000000000011111111111,d0	; clear existing address
				or.l	d2,d0

				move.l	d0,(a0)								; update object GFX pointer
				
               rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; set object pointed to by (a0) to new x/y position
;;
;;
;; a0=object list data
;; d0=x
;; d1=y
				
setup_object_xy:
				add.w	left_marg,d0
				and.w	#%111111111111,d0
				and.w	#%1111111111,d1
				move.w  6(a0),d4                			; get y-pos
                and.w   #$c007,d4
                move.w  14(a0),d6               			; get x-pos
                and.w   #$f000,d6
                add.w   d1,d1
                lsl.w   #3,d1
                or.w    d1,d4
                or.w    d0,d6
                move.w  d4,6(a0)							; update in object data
                move.w  d6,14(a0)
				rts
								
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Wait for a VBL
;;

wait_frame:		move.w	d7,-(a7)
                move.w  ticks,d7
.wait:          cmp.w   ticks,d7
                beq.s   .wait
				move.w (a7)+,d7
                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Vertical Blank Interupt (At least it happens every frame somewhere......)
;;

VB_INTERUPT:    movem.l d0-d7/a0-a6,-(a7)

; patch up the branches BEFORE we copy the OL

				lea 	w1_branch_add,a0

				tst.l	thread1+thread_active
				bpl.s	.t1_active
				move.l	8(a0),d1
				bra.s	.t1_set
.t1_active:		move.l	4(a0),d1
.t1_set:		move.l	(a0),a1
				lea		12(a0),a0
				bsr		.set_branch
				
				tst.l	thread2+thread_active
				bpl.s	.t2_active
				move.l	8(a0),d1
				bra.s	.t2_set
.t2_active:		move.l	4(a0),d1
.t2_set:		move.l	(a0),a1
				lea		12(a0),a0
				bsr		.set_branch
				
				tst.l	thread3+thread_active
				bpl.s	.t3_active
				move.l	8(a0),d1
				bra.s	.t3_set
.t3_active:		move.l	4(a0),d1
.t3_set:		move.l	(a0),a1
				lea		12(a0),a0
				bsr		.set_branch
				
				tst.l	thread5+thread_active
				bpl.s	.t5_active
				move.l	8(a0),d1
				bra.s	.t5_set
.t5_active:		move.l	4(a0),d1
.t5_set:		move.l	(a0),a1
				lea		12(a0),a0
				bsr		.set_branch
				
				tst.l	thread6+thread_active
				bpl.s	.t6_active
				move.l	8(a0),d1
				bra.s	.t6_set
.t6_active:		move.l	4(a0),d1
.t6_set:		move.l	(a0),a1
				lea		12(a0),a0
				bsr		.set_branch
				
                jsr     copy_olist              			; use Blitter to update active list from shadow

				tst.w	shoot_delay
				bmi.s	.tickss
				sub.w	#1,shoot_delay
.tickss:        sub.w   #1,ticks                			; new frame....
				
				tst.w	explode_sfx_timer
				bmi.s	.expld_ss
				sub.w	#1,explode_sfx_timer
.expld_ss:		
				
				sub.w	#1,ebar_ticks
				jsr		musicplayer+8
				
.done_vbl:
				tst.w	shield_stayon
				bmi.s	.do_shld_rech
				sub.w	#1,shield_stayon
.do_shld_rech:	tst.w	shield_recharge
				bmi.s	.do_map_animation
				sub.w	#1,shield_recharge
.do_map_animation:

				tst.l	do_pause_scroller
				bmi.s	.no_paused
				bsr		pause_scroller
.no_paused:	
				sub.w	#1,pause_ticker

				sub.l	#1,map_anim_swap
				bpl.s	.no_swap_map
				move.l	map_anim_speed,map_anim_swap
				lea		current_map,a0
				movem.l	(a0),d0-d3
				move.l	d0,d4
				move.l	d1,d0
				move.l	d2,d1
				move.l	d3,d2
				move.l	d4,d3
				movem.l	d0-d3,(a0)
.no_swap_map:

				sub.w	#1,.swapwave_counter
				bpl.s	.exit_vbi
				move.w	#10,.swapwave_counter

				lea		.leftwave_bullets(pc),a5
				lea		.ritewave_bullets(pc),a6
.getagain:		move.w	.offset_bullets,d0
				move.l	(a5,d0.w),d1
				move.l	(a6,d0.w),d2
				bpl.s	.its_ok
				clr.w	.offset_bullets
				bra		.getagain
.its_ok:		add.w	#4,.offset_bullets	

				lea		E_0AC,a0
				lea		E_0AD,a1
				move.l	d1,etype_bulltype(a0)
				move.l	d2,etype_bulltype(a1)

				sub.w	#1,.fireframe_counter
				bpl.s	.exit_vbi
				move.w	#10,.fireframe_counter
				
				lea		.fireframelist(pc),a5
				move.l	(a5,d0.w),d3
				cmp.l	#10,d3
				bne.s	.argh
				clr.l	.fireframe_counter
.argh:			move.l	d3,etype_1stfire(a0)
				move.l	d3,etype_1stfire(a1)
				
.exit_vbi:		move.w  #$101,INT1              			; Signal we're done
                move.w  #$0,INT2
                
                movem.l (a7)+,d0-d7/a0-a6
                rte
			
.offset_bullets:	dc.w	0			
.leftwave_bullets:	dc.l	2,4,2,10,-1
.ritewave_bullets:	dc.l	11,2,4,2,-1
.swapwave_counter:	dc.w	10

.fireframe_counter:	dc.w	0

.fireframelist:		dc.l	999,999,999,10

;; addr of object in a1
;; shifted address of jump in d1
				
.set_branch:	move.l	2(a1),d6
				and.l 	#%11111000000000000000000011111111,d6
				or.l	d1,d6
				move.l	d6,2(a1)
				rts
				
left_edge:		dc.l 0


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Pallette Routine: fade to black
;;

fadepal_down:   move.w  #32,d6
.dofader:       lea     $f00400,a0      ; hardware pallette
                move.w  #255,d7         ; 254 colours
.palloop:       move.w  (a0),d0     
                move.w  d0,d1
                move.w  d0,d2
                and.w   #%1111100000000000,d0
                and.w   #%0000011111000000,d1
                and.w   #%0000000000111110,d2
.d0:            tst.w   d0
                beq.s   .d1
                sub.w   #%0000100000000000,d0
.d1:            tst.w   d1
                beq.s   .d2
                sub.w   #%0000000001000000,d1
.d2:            tst.w   d2
                beq.s   .merge
                sub.w   #%0000000000000010,d2
.merge:         or.w    d1,d0
                or.w    d2,d0
                move.w  d0,(a0)+
                dbf     d7,.palloop

				movem.l	d0-d7/a0-a6,-(a7)
				lea		GPU_tilemap_update,a0
				bsr		call_GPU_code		; YEAH!!!!
                bsr     wait_frame
				lea		GPU_tilemap_update,a0
				bsr		call_GPU_code		; YEAH!!!!
                bsr     wait_frame
				movem.l	(a7)+,d0-d7/a0-a6

                dbf     d6,.dofader
                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Pallette Routine: fade from black to required pallette
;;

fadepal_up:     move.w  #32,d6
                move.l  a1,a4
.dofader2:      lea     $f00400,a0      ; hardware pallette
                move.l  a4,a1
                move.w  #255,d7         ; 254 colours
.palloop2:      move.w  (a0),d0         ; word in pallette     
                move.w  (a1)+,d3        ; required word
                move.w  d0,d1
                move.w  d0,d2
                move.w  d3,d4
                move.w  d3,d5
                and.w   #%1111100000000000,d0
                and.w   #%0000011111000000,d1
                and.w   #%0000000000111110,d2
                and.w   #%1111100000000000,d3
                and.w   #%0000011111000000,d4
                and.w   #%0000000000111110,d5
.d0_1:          cmp.w   d0,d3
                beq.s   .d1_2
                add.w   #%0000100000000000,d0
.d1_2:          cmp.w   d1,d4
                beq.s   .d2_3
                add.w   #%0000000001000000,d1
.d2_3:          cmp.w   d2,d5
                beq.s   .merge_2
                add.w   #%0000000000000010,d2
.merge_2:       or.w    d1,d0
                or.w    d2,d0
                move.w  d0,(a0)+
                dbf     d7,.palloop2
                bsr     wait_frame
				bsr		map_anim
                bsr     wait_frame
				bsr		map_anim
				bsr		wait_frame
				bsr		map_anim
                dbf     d6,.dofader2
                rts
				
map_anim:		lea		GPU_tilemap_update,a0
				bsr		call_GPU_code		
				lea		GPU_restore_backdrop,a0
				bsr		call_GPU_code
				lea		GPU_save_and_plot,a0
				bsr		call_GPU_code
				rts
				
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Initialise table of object tile pointers
;;
				
Create_TilePointerTable:
				move.l	#maptiles,a0
				lea		tile_pointers,a1
				move.l	#tileset_y-1,d7						; vertical blocks
.vertical:		move.l	#tileset_x-1,d6						; horizontal blocks
.horizontal:	
				move.l	a0,(a1)+							; store gfx address
				lea		32(a0),a0							; offset to next tile (H)
				dbra 	d6,.horizontal
				lea		320*31(a0),a0						; offset to next tile (V)
				dbra	d7,.vertical
				rts

				include "_JOYPAD.S"						; Jagpad stuffs
				include "_MAKEOL.S"						; make object list
				include "_INITBMP.S"					; convert BMPtoJAG
				include "_INITOBJ.S"					; init objectdata
				include "_INITVID.S"					; init video system
				include "_TEXTOUT.S"					; scoreboard stuffs
				include "_PAUSE.S"						; pause mode
				include "BADMOOD/BADMOODI.S"
					
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Clear ram routine
;;

clear_memory:	clr.b	(a0)+
				cmp.l	a0,a1
				bne.s	clear_memory
				rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Copy ram routine
;;

copy_ram:       move.w (a0)+,(a2)+
                cmp.l a0,a1
                bne.s copy_ram
                rts


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Copy GPU code to GPU local
;;				
				
install_GPU_code:
                move.l  B_CMD,d0                ; make sure blitter
                btst    #0,d0                   ; is done
                beq.s   install_GPU_code
				
                lea     GPU_START,a0      ; start of GPU code
				lea		$f03080,a1
				move.l	#(4096-$80/4)-1,d7
.gpuup:			move.l	(a0)+,(a1)+
				dbra	d7,.gpuup
				rts
								
				.phrase

GPU_START:		.gpu
				.org	$f03080
				
GPU_EXEC:
				include "_GPUTIL3.S"	; GPU tilemap update
				include "_GPUBULT.S"	; GPU bullet movement
				include "_GPUSHIP.S"	; GPU process player ship
				include "_GPUADD2.S"	; GPU add bullet
				include "_GPUANIM.S"	; GPU Animation/Wave Processing
				include "_GPUCOLI.S"	; GPU collision detection
				include "_GPUBSRT.S"	; GPU Bullet sorting
				include "_GPUBOMB.S"	; GPU SmartBomb
				include "_GPUSTAR.S"	; GPU parallax starfield
				
				.68000
				.phrase
				
GPU_END:		.dc.l	0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Execute code on GPU
;;	

call_GPU_code:	move.l	a0,G_PC
                move.l  #GPUGO,G_CTRL           ; Start GPU

.wait4flash:    move.l  G_CTRL,d0               ; Wait for complete
                andi.l  #$1,d0
                bne.s   .wait4flash
				
				rts

call_GPU_code_music:
				move.l	a0,G_PC
                move.l  #GPUGO,G_CTRL           ; Start GPU
				jsr		musicplayer+8
.wait4flash:    move.l  G_CTRL,d0               ; Wait for complete
                andi.l  #$1,d0
                bne.s   .wait4flash
				
				rts

			
Init_music:     jsr     musicplayer
				move.l	$1100,d0
                jsr     musicplayer+$14              ; mid range volume
                move.l	$1104,d0
                jsr     musicplayer+$18
				move.l	$1008,a0
				add.l	#alerted_mod_off,a0
				jsr     musicplayer+4
				rts
				
				include "_AUDIO.S"
				
weapon_power:			.dc.l	6
explode_sfx_trigger:	.dc.l 0
smartbomb_sfx_trigger:	.dc.l	0

				.phrase
explode_sfx_timer:		.dc.w -1
dead_sfx_timer:			.dc.w -1
				.phrase
map_x_pos:		.dc.w 	0										; current map screen display (x in pixels)
map_y_pos:		.dc.w 	(mapheight*32)-256						; current map screen display (y in pixels)
map_y_cnt:		.dc.w 	(mapheight*32)-256						; counter to end of map
map_y_dir:		.dc.w 	-1										; scroll direction (-1 = up / 1=down)

map_y_cnt_save:		.dc.w 0
map_y_pos_save:		.dc.w 0
map_y_dir_save:		.dc.w 0
wave_event_save:	.dc.l 0

				.phrase

player_hit_flag:	.dc.l	0
bonus_hit_flag:		.dc.l	0
bonus_hit_frame:	.dc.l	0

ship_x:			.dc.w	0									; player ship x-co-ord
ship_y:			.dc.w	0									; player ship y-co-ord
ebar_x:			.dc.w	0
ebar_y:			.dc.w	0
ebar_ticks:		.dc.w	0

				.phrase
left_marg:		.dc.w 	0
frcount: 		.dc.w 	0

ship_anim_shadow:	.dc.l	0				
				
;;;;;;;;;;;;;;
;;;;;;;;;;;;;; ENEMY TYPE DATA
;;;;;;;;;;;;;;

				.phrase
				
thread_lookup:	.dc.l	thread1
				.dc.l	thread2
				.dc.l	thread3
				.dc.l	thread4
				.dc.l	thread5
				.dc.l	thread6
				
				.dc.l	0,0,0,0
		
				.phrase
thread1:		.dc.l	e_thread1,0,0,0,0,0,0,0,0
				.phrase
thread2:		.dc.l	e_thread2,0,0,0,0,0,0,0,0
				.phrase
thread3:		.dc.l	e_thread3,0,0,0,0,0,0,0,0
				.phrase
thread4:		.dc.l	e_thread4,0,0,0,0,0,0,0,0
				.phrase
thread5:		.dc.l	e_thread5,0,0,0,0,0,0,0,0
				.phrase
thread6:		.dc.l	e_thread6,0,0,0,0,0,0,0,0
					
					
				.phrase
				include "L1WAVES/_WAVETRG.S"
				include "L1BOSS/_BOSSTRG.S"
				include "L1BOSS/_BOSSDIE.S"
				include "_ENEMY.S"
				include "_BULLET.S"
				
EOF_WAVE:		dc.l $03e703e7,$03e703e7
				
map_anim_swap:	dc.l	3
map_anim_speed:	dc.l	3
				
				.phrase
current_map:	dc.l	map_data,map_data2,map_data3,map_data4
				.phrase
map_data:		include "MAPDATA/final1.S"
				.phrase
map_data2:		include "MAPDATA/final2.s"
				.phrase
map_data3:		include "MAPDATA/final3.s"
				.phrase
map_data4:		include "MAPDATA/final4.s"
				.phrase
map_data_end:	dc.b	"EMAP"

				.phrase
gameover_txt_bmp:	
				.rept	40*8
				dc.b 0
				.endr

				.phrase
starbuffer:		
				.rept	max_stars*2
				dc.l	0
				.endr
				
				.phrase
starpointer:	incbin "MAPDATA/STARS2.XYS"
				.phrase
				
underlay_bmp:	incbin "JAGBMPS/NEWSTARS.JAG"
				.phrase
maptiles:		incbin "MAPDATA/FINAL.JAG"
				.phrase
bullet00_bmp:	incbin "JAGBMPS/BULLETS3.JAG"
				.phrase
laser00_bmp:	incbin "JAGBMPS/LASER.JAG"
				.phrase
playership_bmp:	incbin "JAGBMPS/FUJI24.J16"
				.phrase
toppanel_bmp:	incbin "JAGBMPS/PANEL.JAG"
				.phrase
botpanel_bmp:	incbin "JAGBMPS/PROJECT1.J16"
				.phrase
lifebar_bmp:	incbin "JAGBMPS/ENERGYBR.JAG"
				.phrase
bonus_bmp:		incbin "JAGBMPS/BONUS2.JAG"
				incbin "ENEMIES/ENDBOSS/BLOB.J16"
				.phrase
fujishld_bmp:	incbin "JAGBMPS/FUJISHLD.JAG"
				.phrase
shipdead_bmp:	incbin "JAGBMPS/shitBOOM.J16"
				.phrase
gameover_bmp:	incbin "JAGBMPS/GAMEOVER.J16"
				.phrase
enemy00_bmp:	incbin "ENEMIES/AIR/64A_04B.JAG"
enemy00_pal:	incbin "ENEMIES/AIR/64A_PAL.JAG"
				.phrase
enemy01_bmp:	incbin "ENEMIES/AIR/64B_04B.JAG"
enemy01_pal:	incbin "ENEMIES/AIR/64B_PAL.JAG"
				.phrase
enemy02_bmp:	incbin "ENEMIES/AIR/32A_04B.JAG"
enemy02_pal:	incbin "ENEMIES/AIR/32A_PAL.JAG"
				.phrase
enemy03_bmp:	incbin "ENEMIES/AIR/16A_04B.JAG"
enemy03_pal:	incbin "ENEMIES/AIR/16A_PAL.JAG"
				.phrase
bosshat_bmp:	incbin "ENEMIES/ENDBOSS/HELMET.JAG"
bosshat_pal:	incbin "ENEMIES/ENDBOSS/HELMET.PAL"
				.phrase
enemy07_bmp:	incbin "ENEMIES/GND/45L_04B.JAG"
enemy07_pal:	incbin "ENEMIES/GND/45L_PAL.JAG"
				.phrase
enemy08_bmp:	incbin "ENEMIES/GND/45R_04B.JAG"
enemy08_pal:	incbin "ENEMIES/GND/45R_PAL.JAG"
				.phrase
enemy09_bmp:	incbin "ENEMIES/AIR/16B_04B.JAG"
enemy09_pal:	incbin "ENEMIES/AIR/16B_PAL.JAG"
				.phrase
enemy0A_bmp:	incbin "ENEMIES/AIR/16C_04B.JAG"
enemy0A_pal:	incbin "ENEMIES/AIR/16C_PAL.JAG"
				.phrase
enemy0B_bmp:	incbin "ENEMIES/GND/GRDB_04B.JAG"
enemy0B_pal:	incbin "ENEMIES/GND/GRDB_PAL.JAG"
				.phrase
				
midboss01_bmp:	incbin "ENEMIES/MIDBOSS/MIDB_04B.JAG"
midboss01_pal:	incbin "ENEMIES/MIDBOSS/MIDB_PAL.JAG"
				.phrase
midboss02_bmp:	incbin "ENEMIES/MIDBOSS/PROT_04B.JAG"
midboss02_pal:	incbin "ENEMIES/MIDBOSS/PROT_PAL.JAG"
				.phrase
				

;all_samps:		incbin "SAMPLES/ALLSAMPS.JAG"
;				.phrase
				
;shipdead:		incbin "SAMPLES/SHIPDEAD.RAW"
;				.phrase
;gunshot:		incbin "SAMPLES/FASTGUN.RAW"	
;				.phrase
;lasshot:		incbin "SAMPLES/JACK.RAW"
;				.phrase
;explode:		incbin "SAMPLES/EXPLODE.RAW"	
;				.phrase
;gun_up:			incbin "SAMPLES/1BLASTER.RAW"
;				.phrase
;laser_up:		incbin "SAMPLES/1LASER.RAW"
;				.phrase
;laser_down:		incbin "SAMPLES/1LASERD.RAW"
;				.phrase
;smart_up:		incbin "SAMPLES/1SMART.RAW"
;				.phrase
;shield_act:		incbin "SAMPLES/1SHIELDA.RAW"
;				.phrase
;shield_dea:		incbin "SAMPLES/1SHIELDD.RAW"
;				.phrase
;getready_sam:	incbin "SAMPLES/1READY.RAW"
;				.phrase
;gameover_sam:	incbin "SAMPLES/1GAMEOVR.RAW"
;				.phrase
;smart_fire:		incbin "SAMPLES/SMRTBOMB.RAW"
;				.phrase
				
;module:			incbin "SNDTRACK/S47_2.MOD"
;				.phrase
pallette:
				incbin "MAPDATA/THE_PAL.BMP"		
				.phrase
				
				include	"BADMOOD/BAD_DECS.S"
				
                .bss
                .dphrase

bss_top:		.ds.l 	2

				.phrase

bspawn_table:	.ds.l	3*max_bullets			; 3 longs/16 possible new spawn points

e_thread1:		.ds.b	max_enemies1*64

e_thread2:		.ds.b	max_enemies2*64

e_thread3:		.ds.b	max_enemies3*64

e_thread5:		.ds.b	max_enemies5*64

e_thread6:		.ds.b	max_enemies6*64

e_thread4:		.ds.b	max_enemies4*64


bullets:		.ds.b	max_bullets*32
bullet_special:	.ds.b	32
bullet_crudOLD:	.ds.b	32

				.phrase
y060_100:		.ds.b	(max_bullets+2)*4
y101_140:		.ds.b	(max_bullets+2)*4
y141_180:		.ds.b	(max_bullets+2)*4
y181_220:		.ds.b	(max_bullets+2)*4
y221_256:		.ds.b	(max_bullets+2)*4

				.phrase
				
w1_branch_add:	.ds.l	1
w1_branch_no:	.ds.l	1
w1_branch_yes:	.ds.l	1

w2_branch_add:	.ds.l	1
w2_branch_no:	.ds.l	1
w2_branch_yes:	.ds.l	1

w3_branch_add:	.ds.l	1
w3_branch_no:	.ds.l	1
w3_branch_yes:	.ds.l	1

w5_branch_add:	.ds.l	1
w5_branch_no:	.ds.l	1
w5_branch_yes:	.ds.l	1

w6_branch_add:	.ds.l	1
w6_branch_no:	.ds.l	1
w6_branch_yes:	.ds.l	1
		
ship_anim:		.ds.l	1
px_now:			.ds.l	1
py_now:			.ds.l	1
py_cnt:			.ds.l	1
px_start:		.ds.l	1
py_start:		.ds.l	1
life_energy:	.ds.l	1
lives:			.ds.l	1

branch60:		.ds.l	1
branch100:		.ds.l	1
branch140:		.ds.l	1
branch180:		.ds.l	1
branch220:		.ds.l	1
branch256:		.ds.l	1
branchend:		.ds.l	1

ebranch60:		.ds.l	1
ebranch100:		.ds.l	1
ebranch140:		.ds.l	1
ebranch180:		.ds.l	1
ebranch220:		.ds.l	1
ebranch256:		.ds.l	1
end_of_enemies:	.ds.l	1

lives_obj:		.ds.l	4
ship_obj:		.ds.l	1
lifebar_obj:	.ds.l	1

sideobj:		.ds.l	1
lasrobj:		.ds.l	1
bombobj:		.ds.l	1
shldobj:		.ds.l	1
fujishld_obj:	.ds.l	1
underlay_obj:	.ds.l	1
topmask_obj:	.ds.l	1
botmask_obj:	.ds.l	1
skip_all_bullets_obj:	.ds.l	1
mapob_ptrs:		.ds.l   maptile_x*maptile_y

tile_pointers:	.ds.l   800					; 1 long per tile
				
game_pal:		.ds.w	256
tmppal:			.ds.w 	256

ship_anim_tab:	.ds.l	8

ticks:          .ds.l   1                       ; Incrementing # of ticks
shoot_delay:	.ds.w	1
ntsc_flag:		.ds.w	1

a_hdb:          .ds.w   1
a_hde:          .ds.w   1
a_vdb:          .ds.w   1
a_vde:          .ds.w   1
width:          .ds.w   1
height:         .ds.w   1
  
				.phrase
_trashram:		

				.end




