;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; GPU Animation/Thread Management code
;;

;
;	Register Bank 1
;	R00=enemies in wave
;	R01=bullet spawn table
;
;	R02=current animation frame
;	R03=end check animation frame
;	R04=frame to display
;
;	R05=OL Mask
;	R06=OL Mask
;	R07=OL Mask
;	R08=OL Mask
;	R09=OL Mask
;
;	R10=1st frame of shooting animation
;	R11=JUMP: .animator
;
;	R12=next frame to display			
; 	
;	R14=index to per enemy table
;	R15=RESERVED (2nd index lookup)
;
;	R16=JUMP: .mainloop
;	R17=JUMP: .next_enemy
;
;	R19=enemy action
;
;	R20=GFX pointer
;	R21=bytes per frame
; 	R22=animation speed
;	R23=end of main anim
;	R24=hit frame number
;	R25=first firing frame
;	R26=last firing frame
;	R27=bullet generation frame
;	R28=bullet type
;	R29=bullet spawn X offset
;	R30=bullet spawn Y offset
;	R31=Left margin offset
;
;	Register Bank 2
;
;	
;	R01	Enemy action #2 (collision flag)
;	R02	pointer to thread table
;	R04	#0000FFFF
;	R05 E-Type Animloop
;	R06 E-Type looped speed
;	R07	E-Type Groundflag
;	R08 Enemies in wave (remove check)
;	R09 shadow thread pointer
;	R10	Enemy killframe
;	R11 Enemy destroyed frame
;
;	R19: Thread Reset pointer
;	R20 J: .shooting_code
; 	R21 J: .destroyed_code_init
;	R22 J: .destroyed_code_anim
;
;	R14 copy of R14
;	R31	map x-pos
;

; for each thread

thread6_GPU:
		movei	#thread_common,r21
		movei	#thread6,r15					; get thread table pointer
		jump	(r21)
		nop

thread5_GPU:
		movei	#thread_common,r21
		movei	#thread5,r15					; get thread table pointer
		jump	(r21)
		nop

thread1_GPU:
		movei	#thread_common,r21
		movei	#thread1,r15					; get thread table pointer
		jump	(r21)
		nop

thread2_GPU:
		movei	#thread2,r15					; get thread table pointer
		jr		thread_common
		nop
		
thread3_GPU:
		movei	#thread3,r15					; get thread table pointer
		jr		thread_common
		nop
		
thread4_GPU:
		movei	#thread4,r15
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Common For each thread:
;;

thread_common:
		load	(r15),r14								; get e_thread pointer
		movei	#$ffff,r20								; mask
		load	(r15+(thread_ecounter/4)),r0			; number of enemies in wave
		load	(r15+(thread_etype/4)),r1				; enemy type
		load	(r15+(thread_reset/4)),r2
		moveta	r14,r2									; backup table pointer
		moveta	r15,r9
		moveta	r20,r4
		moveta	r2,r19
;		we are now finished with the #thread tables, r15 can be re-used
		
		movei	#enemy_lookup,r15
		shlq	#2,r1									; mult etype*4
		load	(r15+r1),r15							; r15=pointer to enemy type
		moveta	r0,r8									; copy enemies in wave to backup (for removal check)	
		movei	#.shooting_code,r20
		movei	#.destroyed_code_init,r21				; pre-load JUMP addresses to alt-bank
		movei	#.destroyed_code_anim,r22
		moveta	r20,r20
		moveta	r21,r21
		moveta	r22,r22
		load	(r15+(etype_animloop/4)),r26
		load	(r15+(etype_movedead/4)),r10
		load	(r15+(etype_loopdelay/4)),r27
		load	(r15+(etype_gfxptr/4)),r20				; R20=GFX pointer
		moveta	r10,r7
		load	(r15+(etype_bytes_nf/4)),r21			; R21=bytes per frame
		load	(r15+(etype_animspd/4)),r22				; R22=animation frame speed
		load	(r15+(etype_mainanim/4)),r23			; R23=end frame of main anim
		moveta	r26,r5
		moveta	r27,r6
		load	(r15+(etype_hitframe/4)),r24			; R24=hit frame number
		load	(r15+(etype_shootfrm/4)),r10			; R10=frame of main anim to trigger firing anim
		load	(r15+(etype_1stfire/4)),r25				; R25=first frame of fire animation
		load	(r15+(etype_fireanim/4)),r26			; R26=last frame of firing anim
		load	(r15+(etype_bullgen/4)),r27				; R27=bullet generation frame
		load	(r15+(etype_bulltype/4)),r28			; R28=bullet type to spawn
		movei	#map_x_pos,r3
		loadw	(r3),r31
		shlq	#16,r31
		shrq	#16,r31
		moveta	r31,r31									; store in alt-bank
		load	(r15+(etype_bxoff/4)),r29				; R29=bullet spawn X
		load	(r15+(etype_byoff/4)),r30				; R30=bullet spawn Y
		load	(r15+(etype_killframe/4)),r3
		load	(r15+(etype_deadframe/4)),r4
		moveta	r3,r10
		moveta	r4,r11
		movei	#left_marg,r3							; R31=Left margin offset
		moveq	#0,r31
		loadw	(r3),r31
		movei	#%1111111111,r5							; Object list masks
		movei	#%111111111111,r6
		movei	#$ffffc007,r7
		movei	#$fffff000,r8
		movei	#%11111111111,r9		
		movei	#bspawn_table,r1						; R01=bullet spawn table

;		* for each enemy in wave

		movei	#.mainloop,r16
		movei	#.next_enemy,r17
		movei	#.animator,r11
.mainloop:
		load	(r14+(enemy_action2/4)),r19				; load ALT enemy action flag
		load	(r14+(enemy_curframe/4)),r2				; R02=current animation frame

		moveta	r19,r1
		
		load	(r14+(enemy_use/4)),r19					; R19=enemy use
		cmpq	#1,r19
		jump	NE,(r17)								; R19=-ve - enemy unused
		moveq	#15,r18				; *** FLAG THIS COUNTS AS DESTROYED ***
		
		load	(r14+(enemy_initdelay/4)),r13
		subq	#1,r13
		store	r13,(r14+(enemy_initdelay/4))
		JUMP	PL,(r17)								; next enemy if not ready
		
		load	(r14+(enemy_action/4)),r19
		nop
		
		cmpq	#2,r19									; R19=5, death animation start
		jr		EQ,.destroyed_anim_init
		nop

		cmpq	#9,r19									; R19=9, death animating
		jr		EQ,.destroyed_anim_play
		nop

		cmpq	#3,r19									; R19=3, shooting
		jr		EQ,.shooting_anim
		nop	

		jr		.normal_anim
		nop

.shooting_anim:											; jump code to get around relative
		movefa	r20,r18									; branching limitations and not
		jump	(r18)		; * .shooting_code			; eat registers
		nop
.destroyed_anim_init:
		movefa	r21,r18		; * .destroyed_code_init
		jump	(r18)
		nop
.destroyed_anim_play:
		movefa	r22,r18		; *.destroyed_code_anim
		jump	(r18)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Normal Animation Mode
;;
.normal_anim:
		move	r23,r3									; load normal-end frame to Eo-animation check
		cmp		r2,r25									; fire frame?
		JUMP	NE,(r11)								; if not, jump to animation exit	
		nop
		moveq	#3,r18									; set action=fire
		store 	r18,(r14+(enemy_action/4))
		move	r10,r2									; set anim pointer to 1st frame of fire animation
		JUMP	(r11)									; jump to animator
		nop
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Starting to shoot
;;

.shooting_code:
		move	r26,r3									; load firing-end frame to Eo-Anmiation check
		cmp		r2,r27									; check if on "spawn bullet frame"
		JUMP	NE,(r11)								; jump to animator
		nop

		load	(r14+(enemy_shooting/4)),r18
		cmpq	#1,r18
		JUMP	EQ,(r11)

		
		moveq	#1,r18
		store	r18,(r14+(enemy_shooting/4))			; flag as shooting

		load	(r14+(enemy_x/4)),r18					; if so, spawn a bullet of bullet-type
		load	(r14+(enemy_y/4)),r15					;  at x/y and increment spawn table pointer
		add		r29,r18
		add		r30,r15
		store	r18,(r1)
		addq	#4,r1
		store	r15,(r1)
		addq	#4,r1
		store	r28,(r1)
		addq	#4,r1
		addq	#1,r2
		
		JUMP	(r11)									; jump to animator
		nop
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Destroyed 
;;

.destroyed_code_init:
		moveq	#31,r3									; flag animator as NOT HIT
		moveta	r3,r1
		
		moveq	#9,r19
		movefa	r10,r2									; start of killframe
		movefa	r11,r3									; last frame of kill animation
		store	r22,(r14+(enemy_animspd/4))
		store	r19,(r14+(enemy_action/4))				; flag as animating
		JUMP	(r11)									; jump to animator
		nop
		
.destroyed_code_anim:
		moveq	#31,r3									; flag animator as NOT HIT
		moveta	r3,r1

		movefa	r11,r3									; load end of destroyed-frame to Eo-Animation check
		cmp		r2,r3									; check if dead?
		jr		ne,.animator
		nop

		movei	#.remove_enemy,r18
		jump	(r18)
		nop
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Animator Code
;;

.animator:
		move	r2,r12									; next frame to display in R12
		movefa	r1,r18
		cmpq	#7,r18									; was it hit?
		JR		NE,.animate_cont
		nop

		moveq	#1,r15									; change enemy_action so as not to get "stuck"							
		add		r24,r12									; add hit frame offset into next to display
		store	r15,(r14+(enemy_action2/4))

.animate_cont:
		load	(r14+(enemy_obj/4)),r15					; R15= pointer to object list data

		move	r12,r18									; frame to display
		mult	r21,r18									; *bytes per frame
		add		r20,r18									; add base address. R18=graphics address
		
		load	(r15),r12								; LONG with address in
				
		shlq	#8,r18
		and		r9,r12
		or		r18,r12
		store	r12,(r15)								; write new address back to OL
		
		cmp		r2,r3									; end of current animation set?
		JR		NE,.not_end_anim
		nop
		
		movefa	r6,r19
		movefa	r5,r2									; loop frame

		store	r19,(r14+(enemy_animspd/4))				; store new animation speed

		load	(r14+(enemy_shooting/4)),r13			; get shooting action
		cmpq	#1,r13									; was it shooting?
		Jr		NE,.not_end_anim
		nop
		store	r2,(r14+(enemy_shooting/4))				; flag as no longer shooting
			
.not_end_anim:
		load	(r14+(enemy_animspd/4)),r13				; get current counter for animation speed
		subq	#1,r13
		jr		PL,.not_yet								; time to animate?
		nop
		
		addq	#1,r2									; next frame in sequence
		move	r22,r13									; reset animation speed
.not_yet:
		movefa	r7,r18				; get air/ground type
		load	(r14+(enemy_action/4)),r3		; get current object status

		store	r13,(r14+(enemy_animspd/4))
		store	r2,(r14+(enemy_curframe/4))		; store next frame

		cmpq	#0,r18
		jr		NE,.ground_target	; vertically moving ground targets MUST keep moving along wave
		
		cmpq	#9,r3				; animating death atm?
		jump	EQ,(r17)			; yes, jump past waveplotter
.ground_target:	
		load	(r14+(enemy_waveptr/4)),r19		; pointer to pointer for data
		movei	#$03e703e7,r18
		load	(r19),r3
		cmp		r18,r3							; end of wave?
		jr		NE,.not_wave_end				; if yes, remove enemy
		nop
		
		movei	#resume_scrolling,r13			; boss wave?
		load	(r13),r18
		cmpq	#0,r18
		jr		PL,.remove_enemy
		nop

		movefa	r19,r19							; yes, so loop enemy wave
		load	(r19),r3
		
.not_wave_end:
		movefa	r4,r13							; #$ffff
		move	r3,r18							
		shrq	#16,r3							; r3=x.w
		and		r13,r18							; r18=y.w
		
		addq	#4,r19							; increment along wave
		store	r19,(r14+(enemy_waveptr/4))		; store pointer
		jr		.object_update
		nop
		
.remove_enemy:
		movefa	r8,r13
		load	(r14+(enemy_obj/4)),r15
		movei	#-1,r12
		subq	#1,r13							; count down removed
		movei	#700,r18						; set y=offscreen
		store	r12,(r14+(enemy_use/4))			; flag as unused
		move	r18,r3
		moveta	r13,r8							; store back in alt bank
										
.object_update:
		store	r3,(r14+(enemy_x/4))
		store	r18,(r14+(enemy_y/4))
; r3=x
; r18=y
		movefa	r31,r19							; get map x
		sub		r19,r3							; subtract map x-pos
		add		r31,r3							; add left margin
		and		r6,r3							; mask it for negatives
		
		and		r5,r18							; mask y
		add		r18,r18							; double it (interlace)
		
		load	(r15+1),r13						; longword containing Y
		load	(r15+3),r12						; longword containing X		

		and		r8,r12							; mask out old x
		or		r3,r12							; insert new x
	
		shlq	#3,r18
		and		r7,r13							; mask out old y
		or		r18,r13							; insert new y
	
		store	r12,(r15+3)
		store	r13,(r15+1)
		
.next_enemy:								; check if flag
	cmpq	#15,r18							; for "enemy skipped"
	jr 		ne,.no_dec						; and decrement
	nop										; remaining ocunter

	movefa	r8,r12
	subq	#1,r12
	moveta	r12,r8

.no_dec:
		addq	#32,r14
		addq	#32,r14							; next enemy in table
		subq	#1,r0
		cmpq	#0,r0
		JUMP	NE,(r16)						; loop around
		nop

		movei	#-1,r10							; terminate bullet table
		store	r10,(r1)
		addq	#4,r1
		store	r10,(r1)
		addq	#4,r1
		store	r10,(r1)
	
		movefa	r8,r1
		
		nop
		nop
		nop
		
		cmpq	#0,r1				; all enemies removed?
		JR		NE,.term
		nop

		movefa	r9,r0
		
		addq	#thread_active,r0				; terminate thread
		store	r10,(r0)
		
.term:	movei	#stop_gpu,r0
		jump	(r0)							; stop the GPU
		nop

